import aurorax as _aurorax
import datetime as _datetime
from typing import Dict as _Dict


def ephemeris(start_dt: _datetime, end_dt: _datetime, program: str = None, platform: str = None,
              instrument_type: str = None, source_type: str = None, owner: str = None,
              format: str = "basic_info") -> _Dict:
    """
    Retrieve information about the number of existing ephemeris records

    :param start_dt: start date
    :type start_dt: datetime
    :param end_dt: end date
    :type end_dt: datetime
    :param program: program name to filter sources by, defaults to None
    :type program: str, optional
    :param platform: platform name to filter sources by, defaults to None
    :type platform: str, optional
    :param instrument_type: instrument type to filter sources by, defaults to None
    :type instrument_type: str, optional
    :param source_type: source type to filter sources by (heo, leo, lunar, or ground), defaults to None
    :type source_type: str, optional
    :param owner: owner ID to filter sources by, defaults to None
    :type owner: str, optional
    :param format: the format of the ephemeris source returned (identifier_only, basic_info,
                   full_record), defaults to "basic_info"
    :type format: str, optional

    :return: ephemeris availability information
    :rtype: Dict
    """
    params = {
        "start": start_dt.strftime("%Y-%m-%d"),
        "end": end_dt.strftime("%Y-%m-%d"),
        "program": program,
        "platform": platform,
        "instrument_type": instrument_type,
        "source_type": source_type,
        "owner": owner,
        "format": format,
    }
    req = _aurorax.AuroraXRequest(_aurorax.api.URL_EPHEMERIS_AVAILABILITY, params=params)
    res = req.execute()
    return_dict = {
        "status_code": res.status_code,
        "data": res.data
    }
    return return_dict


def data_products(start_dt: _datetime, end_dt: _datetime, program: str = None, platform: str = None,
                  instrument_type: str = None, source_type: str = None, owner: str = None,
                  format: str = "basic_info") -> _Dict:
    """
    Retrieve information about the number of existing data product records

    :param start_dt: start date
    :type start_dt: datetime
    :param end_dt: end date
    :type end_dt: datetime
    :param program: program name to filter sources by, defaults to None
    :type program: str, optional
    :param platform: platform name to filter sources by, defaults to None
    :type platform: str, optional
    :param instrument_type: instrument type to filter sources by, defaults to None
    :type instrument_type: str, optional
    :param source_type: source type to filter sources by (heo, leo, lunar, or ground), defaults to None
    :type source_type: str, optional
    :param owner: owner ID to filter sources by, defaults to None
    :type owner: str, optional
    :param format: the format of the ephemeris source returned (identifier_only, basic_info,
                   full_record), defaults to "basic_info"
    :type format: str, optional

    :return: ephemeris data product information
    :rtype: Dict
    """
    params = {
        "start": start_dt.strftime("%Y-%m-%d"),
        "end": end_dt.strftime("%Y-%m-%d"),
        "program": program,
        "platform": platform,
        "instrument_type": instrument_type,
        "source_type": source_type,
        "owner": owner,
        "format": format,
    }
    req = _aurorax.AuroraXRequest(_aurorax.api.URL_DATA_PRODUCTS_AVAILABILITY, params=params)
    res = req.execute()
    return_dict = {
        "status_code": res.status_code,
        "data": res.data
    }
    return return_dict



import aurorax as _aurorax
from typing import List as _List
from typing import Dict as _Dict


def list(format: str = "basic_info") -> _Dict:
    """
    Retrieves all data source records

    :param format: the format of the data source returned (identifier_only, basic_info,
                   full_record), defaults to "basic_info"
    :type format: str, optional

    :return: information about all data sources
    :rtype: Dict
    """
    # make request
    url = _aurorax.api.URL_DATA_SOURCES
    params = {
        "format": format,
    }
    req = _aurorax.AuroraXRequest(url, params=params)
    res = req.execute()

    # set dict to return
    return_dict = {
        "status_code": res.status_code,
        "data": [],
    }
    if (res.status_code == 200):
        return_dict["data"] = res.data

    # return
    return return_dict


def get_using_identifier(identifier: int, format: str = "basic_info") -> _Dict:
    """
    Retrieves data source record matching identifier

    :param identifier: data source unique identifier
    :type identifier: int
    :param format: the format of the data source returned (identifier_only, basic_info,
                   full_record), defaults to "basic_info"
    :type format: str, optional


    :return: information the specific data source
    :rtype: Dict
    """
    # make request
    url = "%s/%d" % (_aurorax.api.URL_DATA_SOURCES, identifier)
    params = {
        "format": format
    }
    req = _aurorax.AuroraXRequest(url, params=params)
    res = req.execute()

    # set dict to return
    return_dict = {
        "status_code": res.status_code,
        "data": {},
    }
    if (res.status_code == 200):
        return_dict["data"] = res.data

    # return
    return return_dict


def get_using_filters(program: str = None, platform: str = None, instrument_type: str = None, source_type: str = None,
                      owner: str = None, format: str = "basic_info") -> _Dict:
    """
    Retrieves all data source records matching the specified filters

    :param program: program name, defaults to None
    :type program: str, optional
    :param platform: platform name, defaults to None
    :type platform: str, optional
    :param instrument_type: instrument type, defaults to None
    :type instrument_type: str, optional
    :param source_type: source type (leo, heo, lunar, ground), defaults to None
    :type source_type: str, optional
    :param owner: owner account name, defaults to None
    :type owner: str, optional
    :param format: the format of the data source returned (identifier_only, basic_info,
                   full_record), defaults to "basic_info"
    :type format: str, optional

    :return: information about the specific data sources matching the filter criteria
    :rtype: Dict
    """
    # make request
    url = "%s" % (_aurorax.api.URL_DATA_SOURCES)
    params = {
        "program": program,
        "platform": platform,
        "instrument_type": instrument_type,
        "source_type": source_type,
        "owner": owner,
        "format": format,
    }
    req = _aurorax.AuroraXRequest(url, params=params)
    res = req.execute()

    # set dict to return
    return_dict = {
        "status_code": res.status_code,
        "data": [],
    }
    if (res.status_code == 200):
        return_dict["data"] = res.data

    # return
    return return_dict


def get_stats(identifier: int) -> _Dict:
    """
    Retrieves additional statistics about the specified data source such as
    the earliest/latest data record and the total number of data records
    available

    :param identifier: data source identifier
    :type identifier: int

    :return: the data source statistics
    :rtype: Dict
    """
    # make request
    url = "%s/%d/stats" % (_aurorax.api.URL_DATA_SOURCES, identifier)
    req = _aurorax.AuroraXRequest(url)
    res = req.execute()

    # set dict to return
    return_dict = {
        "status_code": res.status_code,
        "data": {},
    }
    if (res.status_code == 200):
        return_dict["data"] = res.data

    # return
    return return_dict


def add(api_key: str, program: str, platform: str, instrument_type: str, source_type: str,
        display_name: str, ephemeris_metadata_schema: _List[_Dict] = [],
        data_products_metadata_schema: _List[_Dict] = [], maintainers: _List = [],
        identifier: int = None) -> _Dict:
    """
    Create a new data source record

    :param api_key: API key associated with your account
    :type api_key: str
    :param program: program name
    :type program: str
    :param platform: platform name
    :type platform: str
    :param instrument_type: instrument type
    :type instrument_type: str
    :param source_type: source type (heo, leo, lunar, ground)
    :type source_type: str
    :param display_name: a more user-friendly display name
    :type display_name: str
    :param ephemeris_metadata_schema: metadata schema, defaults to []
    :type ephemeris_metadata_schema: List[Dict], optional
    :param data_products_metadata_schema: metadata schema, defaults to []
    :type data_products_metadata_schema: List[Dict], optional
    :param maintainers: list of users to give maintainer permissions to, defaults to []
    :type maintainers: List, optional
    :param identifier: data source ID, defaults to None
    :type identifier: int, optional

    :return: the created data source record details
    :rtype: Dict
    """
    # make request
    url = _aurorax.api.URL_DATA_SOURCES
    post_data = {
        "program": program,
        "platform": platform,
        "instrument_type": instrument_type,
        "source_type": source_type,
        "display_name": display_name,
        "ephemeris_metadata_schema": ephemeris_metadata_schema,
        "data_product_metadata_schema": data_products_metadata_schema,
        "maintainers": maintainers,
    }
    if (identifier is not None):
        post_data["identifier"] = identifier
    req = _aurorax.AuroraXRequest(url, method="POST", api_key=api_key, json=post_data)
    res = req.execute()

    # set dict to return
    return_dict = {
        "status_code": res.status_code,
        "data": {},
    }
    if (res.status_code == 200 or res.status_code == 409):
        return_dict["data"] = res.request.json()

    # return
    return return_dict


def delete(api_key: str, identifier: int) -> _Dict:
    """
    Remove a data source record

    :param api_key: API key associated with your account
    :type api_key: str
    :param identifier: unique data source identifier
    :type identifier: int

    :return: status summary
    :rtype: Dict
    """
    # make request
    url = "%s/%s" % (_aurorax.api.URL_DATA_SOURCES, str(identifier))
    req = _aurorax.AuroraXRequest(url, method="DELETE", api_key=api_key)
    res = req.execute()

    # set dict to return
    return_dict = {
        "status_code": res.status_code,
        "data": {},
    }
    if (res.status_code == 409):
        return_dict["data"] = res.request.json()

    # return
    return return_dict


def update(api_key: str, identifier: int, program: str = None, platform: str = None,
           instrument_type: str = None, source_type: str = None, ephemeris_metadata_schema: _List[_Dict] = [],
           data_products_metadata_schema: _List[_Dict] = [], owner: str = None,
           maintainers: _List = None) -> _Dict:
    """
    Update a data source record

    :param api_key: API key associated with your account
    :type api_key: str
    :param program: program name
    :type program: str
    :param platform: platform name
    :type platform: str
    :param instrument_type: instrument type
    :type instrument_type: str
    :param source_type: source type (heo, leo, lunar, ground)
    :type source_type: str
    :param ephemeris_metadata_schema: metadata schema, defaults to []
    :type ephemeris_metadata_schema: List[Dict], optional
    :param data_products_metadata_schema: metadata schema, defaults to []
    :type data_products_metadata_schema: List[Dict], optional
    :param maintainers: list of users to give maintainer permissions to, defaults to None
    :type maintainers: List, optional

    :return: the new data source record details
    :rtype: Dict
    """
    # set new information based on current values and function parameters
    # post_data = {}
    # if (program is not None):
    #     post_data["program"] = program
    # if (platform is not None):
    #     post_data["platform"] = platform
    # if (instrument_type is not None):
    #     post_data["instrument_type"] = instrument_type
    # if (source_type is not None):
    #     post_data["source_type"] = source_type
    # if (ephemeris_metadata_schema is not None):
    #     post_data["ephemeris_metadata_schema"] = ephemeris_metadata_schema
    # if (data_products_metadata_schema is not None):
    #     post_data["data_products_metadata_schema"] = data_products_metadata_schema
    # if (owner is not None):
    #     post_data["owner"] = owner
    # if (maintainers is not None):
    #     post_data["maintainers"] = maintainers

    # # make request
    # url = "%s/%s" % (URL_DATA_SOURCES, str(identifier))
    # print(url)
    # req = AuroraXRequest(url, method="PUT", api_key=api_key, json=post_data)
    # res = req.execute()
    # if (res.status_code != 200 and res.status_code != 409):
    #     res.data = ""
    # elif (res.status_code == 409):
    #     res.data = res.request.json()
    # return res
    pass



import requests as _requests
import pprint as _pprint
from typing import Dict as _Dict

# public globals
_URL_API_STUB = "http://api.staging.aurorax.space"
URL_DATA_SOURCES = "%s/api/v1/data-sources" % (_URL_API_STUB)
URL_EPHEMERIS_AVAILABILITY = "%s/api/v1/availability/ephemeris" % (_URL_API_STUB)
URL_EPHEMERIS_UPLOAD = "%s/api/v1/data-sources/{}/ephemeris" % (_URL_API_STUB)
URL_EPHEMERIS_SEARCH = "%s/api/v1/ephemeris/search" % (_URL_API_STUB)
URL_EPHEMERIS_REQUEST_STATUS = "%s/api/v1/ephemeris/requests/{}" % (_URL_API_STUB)
URL_DATA_PRODUCTS_AVAILABILITY = "%s/api/v1/availability/data_products" % (_URL_API_STUB)
URL_DATA_PRODUCTS_UPLOAD = "%s/api/v1/data-sources/{}/data_products" % (_URL_API_STUB)
URL_DATA_PRODUCTS_SEARCH = "%s/api/v1/data_products/search" % (_URL_API_STUB)
URL_DATA_PRODUCTS_REQUEST_STATUS = "%s/api/v1/data_products/requests/{}" % (_URL_API_STUB)


class AuroraXRequest():
    """
    Class to streamline performing a synchronous or asynchronous request against the AuroraX API
    """

    # private globals
    __STANDARD_REQUEST_HEADERS = {
        "accept": "application/json",
        "Content-Type": "application/json"
    }

    def __init__(self, url: str, params: _Dict = {}, json: _Dict = {}, method: str = "GET", api_key: str = "") -> None:
        """
        AuroraXRequest constructor

        :param url: AuroraX RESTful API endpoint URL
        :type url: str
        :param params: URL parameters to be included in the request
        :type params: Dict, optional
        :param json: JSON data to include in the request (ie. POST data), defaults to {}
        :type json: Dict, optional
        :param method: HTTP method (GET, POST, PUT, DELETE), defaults to "GET"
        :type method: str, optional
        :param api_key: AuroraX API key for endpoints requiring authorization, defaults to ""
        :type api_key: str, optional
        """
        # set attributes
        self.json = json
        self.params = params
        self.method = method.upper()
        self.url = url
        self.api_key = api_key

    def execute(self) -> "AuroraXResponse":
        """
        Initiate ephemeris search request

        :return: AuroraXReponse object for this request
        :rtype: AuroraXResponse
        """
        # prep request headers
        request_headers = self.__STANDARD_REQUEST_HEADERS
        if (self.api_key != ""):
            request_headers["x-aurorax-api-key"] = self.api_key

        # perform request
        req = _requests.request(self.method, self.url, params=self.params, json=self.json, headers=request_headers)

        # create response object
        res = AuroraXResponse(req)

        # return
        return res

    def __str__(self) -> str:
        return self.__repr__()

    def __repr__(self) -> str:
        return _pprint.pformat(self.__dict__)


class AuroraXRawRequest(AuroraXRequest):
    """
    Class to run a raw AuroraX API request and not wrap as an AuroraXResponse object
    """

    # private globals
    __STANDARD_REQUEST_HEADERS = {
        "accept": "application/json",
        "Content-Type": "application/json"
    }

    def execute(self) -> _requests.Request:
        """
        Initiate ephemeris search request

        :return: requests.Request object for this request
        :rtype: requests.Request
        """
        # prep request headers
        request_headers = self.__STANDARD_REQUEST_HEADERS
        if (self.api_key != ""):
            request_headers["x-aurorax-api-key"] = self.api_key

        # perform request
        req = _requests.request(self.method, self.url, params=self.params, json=self.json, headers=request_headers)

        # return
        return req

    def __str__(self) -> str:
        """String method

        :return: string format
        :rtype: str
        """
        return self.__repr__()

    def __repr__(self) -> str:
        """
        Object representation

        :return: object representation
        :rtype: str
        """
        return _pprint.pformat(self.__dict__)


class AuroraXResponse():
    """
    Class containing the response information for an AuroraXRequest
    object.
    """

    def __init__(self, request: AuroraXRequest) -> None:
        """
        Constructor

        :param request: AuroraXResponse object associated with this response
        :type request: AuroraXRequest
        """
        # init values
        self.headers = {}
        self.request = request
        self.data = None
        self.status_code = request.status_code

        # set response values
        self.headers = self.request.headers
        if (self.status_code >= 200 and self.status_code < 300):
            if (self.request.text == ""):
                self.data = ""
            else:
                self.data = self.request.json()
        else:
            self.data = {
                "error": "HTTP error code %d" % (self.status_code),
                "response_body": self.request.text,
            }

    def __str__(self) -> str:
        """String method

        :return: string format
        :rtype: str
        """
        dict = self.__dict__
        if (dict["data"] is not None):
            dict["data"] = "..."
        return _pprint.pformat(dict)

    def __repr__(self) -> str:
        """
        Object representation

        :return: object representation
        :rtype: str
        """
        return "<AuroraXResponse [%d]>" % (self.status_code)


def set_url_stub(stub: str) -> None:
    """
    Change the URL stub for the API. For example if you want to migrate
    data from one endpointt to another.

    :param stub: URL stub (ie. http://api.staging.aurorax.space)
    :type stub: str
    """
    global _URL_API_STUB
    global URL_DATA_SOURCES
    global URL_EPHEMERIS_AVAILABILITY
    global URL_EPHEMERIS_UPLOAD
    global URL_EPHEMERIS_SEARCH
    global URL_EPHEMERIS_REQUEST_STATUS
    global URL_DATA_PRODUCTS_AVAILABILITY
    global URL_DATA_PRODUCTS_UPLOAD
    global URL_DATA_PRODUCTS_SEARCH
    global URL_DATA_PRODUCTS_REQUEST_STATUS

    _URL_API_STUB = stub
    URL_DATA_SOURCES = "%s/api/v1/data-sources" % (_URL_API_STUB)
    URL_EPHEMERIS_AVAILABILITY = "%s/api/v1/availability/ephemeris" % (_URL_API_STUB)
    URL_EPHEMERIS_UPLOAD = "%s/api/v1/data-sources/{}/ephemeris" % (_URL_API_STUB)
    URL_EPHEMERIS_SEARCH = "%s/api/v1/ephemeris/search" % (_URL_API_STUB)
    URL_EPHEMERIS_REQUEST_STATUS = "%s/api/v1/ephemeris/requests/{}" % (_URL_API_STUB)
    URL_DATA_PRODUCTS_AVAILABILITY = "%s/api/v1/availability/data_products" % (_URL_API_STUB)
    URL_DATA_PRODUCTS_UPLOAD = "%s/api/v1/data-sources/{}/data_products" % (_URL_API_STUB)
    URL_DATA_PRODUCTS_SEARCH = "%s/api/v1/data_products/search" % (_URL_API_STUB)
    URL_DATA_PRODUCTS_REQUEST_STATUS = "%s/api/v1/data_products/requests/{}" % (_URL_API_STUB)



import aurorax as _aurorax
import datetime as _datetime
import time as _time
import humanize as _humanize
import pprint as _pprint
from typing import List as _List
from typing import Dict as _Dict
from .requests import STANDARD_POLLING_SLEEP_TIME as _STANDARD_POLLING_SLEEP_TIME


class DataProduct():
    """
    Class representing an AuroraX data product record
    """

    def __init__(self, identifier: int, program: str, platform: str, instrument_type: str,
                 start_dt: _datetime, end_dt: _datetime, url: str, data_product_type: str,
                 metadata: _Dict) -> None:
        """
        Constructor

        :param identifier: data source ID
        :type identifier: int
        :param program: program name
        :type program: str
        :param platform: platform name
        :type platform: str
        :param instrument_type: instrument type name
        :type instrument_type: str
        :param start_dt: timestamp for the beginning of the data product in UTC
        :type start_dt: datetime
        :param end_dt: timestamp for the end of the data product in UTC
        :type end_dt: datetime
        :param url: url of the data product
        :type url: str
        :param data_product_type: type of the data product (keogram, montage, average, summary_plot, movie)
        :type data_product_type: str
        :param metdata: metadata values for this record
        :type metdata: Dict
        """
        self.identifier = identifier
        self.program = program
        self.platform = platform
        self.instrument_type = instrument_type
        self.start_dt = start_dt
        self.end_dt = end_dt
        self.url = url
        self.data_product_type = data_product_type
        self.metadata = metadata

    def to_json_serializable(self) -> _Dict:
        """
        Convert object to a JSON-serializable object (ie. translate datetime
        objects to strings)

        :return: dictionary JSON-serializable object
        :rtype: Dict
        """
        d = self.__dict__

        # reset names of start and end times
        d["start"] = d["start_dt"]
        del d["start_dt"]
        d["end"] = d["end_dt"]
        del d["end_dt"]

        # format start and end times as str
        if (type(d["start"]) is _datetime.datetime):
            d["start"] = d["start"].strftime("%Y-%m-%dT%H:%M:00.000Z")
        if (type(d["end"]) is _datetime.datetime):
            d["end"] = d["end"].strftime("%Y-%m-%dT%H:%M:00.000Z")

        # format metadata
        if (type(self.metadata) is dict):
            for key, value in self.metadata.items():
                if (type(value) is _datetime.datetime or type(value) is _datetime.date):
                    self.metadata[key] = self.metadata[key].strftime("%Y-%m-%dT%H:%M:%S.%f")
        if (type(self.metadata) is list):
            self.metadata = {}

        # return
        return d

    def __str__(self) -> str:
        """String method

        :return: string format
        :rtype: str
        """
        return self.__repr__()

    def __repr__(self) -> str:
        """
        Object representation

        :return: object representation
        :rtype: str
        """
        return _pprint.pformat(self.__dict__)


class Search():
    """
    Class representing an AuroraX data product search
    """

    def __init__(self, start_dt: _datetime, end_dt: _datetime, programs: _List = [], platforms: _List = [],
                 instrument_types: _List = [], data_product_types: _List = [], metadata_filters: _List = []) -> None:
        """
        Create a new Search object

        :param start_dt: start timestamp
        :type start_dt: datetime
        :param end_dt: end timestamp
        :type end_dt: datetime
        :param programs: programs to search through, defaults to []
        :type programs: List, optional
        :param platforms: platforms to search through, defaults to []
        :type platforms: List, optional
        :param instrument_types: instrument types to search through, defaults to []
        :type instrument_types: List, optional
        :param data_product_types: data product types to search through, defaults to []
        :type data_product_types: List, optional
        :param metadata_filters: metadata keys and values to filter on, defaults to []
        :type metadata_filters: List, optional
        """
        self.request = None
        self.request_id = ""
        self.request_url = ""
        self.executed = False
        self.data_available = False
        self.data_url = ""
        self.query = {}
        self.status = {}
        self.data = []
        self.logs = []

        self.start_dt = start_dt
        self.end_dt = end_dt
        self.programs = programs
        self.platforms = platforms
        self.instrument_types = instrument_types
        self.data_product_types = data_product_types
        self.metadata_filters = metadata_filters

    def __str__(self) -> str:
        """String method

        :return: string format
        :rtype: str
        """
        return self.__repr__()

    def __repr__(self) -> str:
        """
        Object representation

        :return: object representation
        :rtype: str
        """
        return _pprint.pformat(self.__dict__)

    def execute(self) -> None:
        """
        Initiate data product search request
        """
        # send request
        url = _aurorax.api.URL_DATA_PRODUCTS_SEARCH
        post_data = {
            "data_sources": {
                "programs": self.programs,
                "platforms": self.platforms,
                "instrument_types": self.instrument_types,
                "data_product_metadata_filters": self.metadata_filters,
            },
            "data_product_type_filters": self.data_product_types,
            "start": self.start_dt.strftime("%Y-%m-%dT%H:%M:%S"),
            "end": self.end_dt.strftime("%Y-%m-%dT%H:%M:%S"),
        }
        self.query = post_data
        req = _aurorax.AuroraXRequest(url, method="POST", json=post_data)
        res = req.execute()

        # set request ID, request_url, executed
        self.executed = True
        if (res.status_code == 202):
            # request successfully dispatched
            self.executed = True
            self.request_url = res.headers["location"]
            self.request_id = self.request_url.rsplit("/", 1)[-1]
        self.request = res

    def update_status(self, status: _Dict = None) -> None:
        """
        Update the status of this data product search request

        :param status: retrieved status (include to avoid requesting it from the API again), defaults to None
        :type status: Dict, optional
        """
        # get the status if it isn't passed in
        if (status is None):
            status = get_request_status(self.request_id)

        # update request status by checking if data URI is set
        if (status["request_status"]["completed"] is True):
            self.data_available = True
            self.data_url = status["request_status"]["data_url"]

        # set class variable "status" and "logs"
        self.status = status
        if (status["status_code"] == 200):
            self.logs = status["data"]["logs"]

    def check_for_data(self) -> None:
        """
        Check to see if data is available for this data product search request
        """
        self.update_status()

    def get_data(self) -> None:
        """
        Retrieve the data available for this data product search request
        """
        if (self.data_url == ""):
            print("No data available, update status first")
            return
        url = self.data_url
        data_res = get_request_data(self.request_id, url=url)
        self.data = data_res["data"]

    def wait(self, poll_interval: float = _STANDARD_POLLING_SLEEP_TIME) -> None:
        """
        Block and wait for the request to complete and data is available for retrieval

        :param poll_interval: time in seconds to wait between polling
                              attempts, defaults to STANDARD_POLLING_SLEEP_TIME
        :type poll_interval: float, optional
        """
        url = _aurorax.api.URL_DATA_PRODUCTS_REQUEST_STATUS.format(self.request_id)
        self.update_status(_aurorax.requests.wait_for_data(url, poll_interval=poll_interval))


def get_metadata_schema(identifier: int) -> _List:
    """
    Get data product metadata schema for a specified data source identifier

    :param identifier: data source ID
    :type identifier: int

    :return: metadata schema
    :rtype: List
    """
    return _aurorax.metadata.get_data_products_schema(identifier)


def get_request_status(request_id: str) -> _Dict:
    """
    Retrieve the request status for a given data product search request ID

    :param request_id: data product search request ID
    :type request_id: str

    :return: status response
    :rtype: Dict
    """
    url = _aurorax.api.URL_DATA_PRODUCTS_REQUEST_STATUS.format(request_id)
    return _aurorax.requests.get_status(url)


def get_request_data(request_id: str, url: str = None) -> _Dict:
    """
    Retrieve the request data for a given data product search request ID

    :param request_id: data product search request ID
    :type request_id: str
    :param url: request data URL, optional (derived if not specified)
    :type url: str, optional

    :return: data response
    :rtype: Dict
    """
    if (url is None):
        url = "%s/data" % (_aurorax.api.URL_DATA_PRODUCTS_REQUEST_STATUS.format(request_id))
    return _aurorax.requests.get_data(url)


def get_request_logs(request_id: str) -> _Dict:
    """
    Retrieve the request logs for a given data product search request ID

    :param request_id: data product search request ID
    :type request_id: str

    :return: logs response
    :rtype: Dict
    """
    url = _aurorax.api.URL_DATA_PRODUCTS_REQUEST_STATUS.format(request_id)
    return _aurorax.requests.get_logs(url)


def wait_for_data(request_id: str, poll_interval: float = _STANDARD_POLLING_SLEEP_TIME) -> _Dict:
    """
    Block and wait for the data to be made available for a given data product search request ID

    :param request_id: data product search request ID
    :type request_id: str
    :param poll_interval: time in seconds to wait between polling
                          attempts, defaults to STANDARD_POLLING_SLEEP_TIME
    :type poll_interval: float, optional

    :return: status response
    :rtype: Dict
    """
    url = _aurorax.api.URL_DATA_PRODUCTS_REQUEST_STATUS.format(request_id)
    return _aurorax.requests.wait_for_data(url, poll_interval=poll_interval)


def search_async(start_dt: _datetime, end_dt: _datetime, programs: _List = [], platforms: _List = [],
                 instrument_types: _List = [], data_product_types: _List = [], metadata_filters: _List = [],
                 show_progress: bool = False) -> _Dict:
    """
    Submit a request for a data product search

    :param start_dt: start timestamp
    :type start_dt: datetime
    :param end_dt: end timestamp
    :type end_dt: datetime
    :param programs: programs to search through, defaults to []
    :type programs: List, optional
    :param platforms: platforms to search through, defaults to []
    :type platforms: List, optional
    :param instrument_types: instrument types to search through, defaults to []
    :type instrument_types: List, optional
    :param data_product_types: data product types to search through (choose from keogram,
                               montage, average, summary_plot, movie), defaults to []
    :type data_product_types: str
    :param metadata_filters: metadata keys and values to filter on, defaults to []
    :type metadata_filters: List, optional
    :param show_progress: show the progress of the request using the request log, defaults to False
    :type show_progress: bool, optional

    :return: request response
    :rtype: Dict
    """
    return search(start_dt,
                  end_dt,
                  programs=programs,
                  platforms=platforms,
                  instrument_types=instrument_types,
                  data_product_types=data_product_types,
                  metadata_filters=metadata_filters,
                  show_progress=show_progress,
                  async_return=True)


def search(start_dt: _datetime, end_dt: _datetime, programs: _List = [], platforms: _List = [],
           instrument_types: _List = [], data_product_types: _List = [], metadata_filters: _List = [],
           show_progress: bool = False, async_return: bool = False,
           poll_interval: float = _STANDARD_POLLING_SLEEP_TIME) -> _Dict:
    """
    Search for data product records

    :param start_dt: start timestamp
    :type start_dt: datetime
    :param end_dt: end timestamp
    :type end_dt: datetime
    :param programs: programs to search through, defaults to []
    :type programs: List, optional
    :param platforms: platforms to search through, defaults to []
    :type platforms: List, optional
    :param instrument_types: instrument types to search through, defaults to []
    :type instrument_types: List, optional
    :param data_product_types: data product types to search through (choose from keogram,
                               montage, average, summary_plot, movie), defaults to []
    :type data_product_types: str
    :param metadata_filters: metadata keys and values to filter on, defaults to []
    :type metadata_filters: List, optional
    :param show_progress: show the progress of the request using the request log, defaults to False
    :type show_progress: bool, optional
    :param async_return: return immediately after sending the request, don't wait for data, defaults to False
    :type async_return: bool, optional
    :param poll_interval: seconds to wait between polling calls, defaults to STANDARD_POLLING_SLEEP_TIME
    :type poll_interval: float, optional

    :return: data product data response; or request response if "async_return" is True
    :rtype: Dict
    """
    # init return dict
    return_dict = {
        "status_code": None,
        "data": [],
    }

    # create a Search() object
    s = Search(start_dt, end_dt, programs, platforms, instrument_types, data_product_types, metadata_filters)
    if (show_progress is True):
        print("[%s] Search object created" % (_datetime.datetime.now()))

    # execute the search
    s.execute()
    if (show_progress is True):
        if (s.executed is True):
            print("[%s] Request submitted" % (_datetime.datetime.now()))
            print("[%s] Request ID: %s" % (_datetime.datetime.now(), s.request_id))
            print("[%s] Request details available at: %s" % (_datetime.datetime.now(), s.request_url))
        else:
            print("[%s] Request failed to submit" % (_datetime.datetime.now()))
            return_dict["status_code"] = s.request.status_code
            return_dict["search_object"] = s
            return return_dict

    # check if async return is specified
    if (async_return is True):
        return_dict["status_code"] = s.request.status_code
        return_dict["search_object"] = s
        if (show_progress is True):
            print("[%s] Async return specified, returning immediately" % (_datetime.datetime.now()))
        return return_dict

    # check the request URL for the status of the request (poll for data)
    first_followup = True
    while (s.data_available is False):
        if (first_followup is True):
            _time.sleep(_aurorax.requests.FIRST_FOLLOWUP_SLEEP_TIME)
            first_followup = False
        else:
            _time.sleep(poll_interval)
        if (show_progress is True):
            print("[%s] Checking for data ..." % (_datetime.datetime.now()))
        s.check_for_data()

    # get the data
    if (show_progress is True):
        print("[%s] Request has data available, retrieving it ..." % (_datetime.datetime.now()))
    s.get_data()

    # return response with the data
    return_dict["status_code"] = s.request.status_code
    return_dict["search_object"] = s
    return_dict["data"] = s.data
    if (show_progress is True):
        print("[%s] Retrieved %s of data containing %d records, completed "
              "search request" % (_datetime.datetime.now(),
                                  _humanize.filesize.naturalsize(s.status["data"]["search_result"]["file_size"]),
                                  s.status["data"]["search_result"]["result_count"]))
    return return_dict


def upload(api_key: str, identifier: int, records: _List["DataProduct"]) -> _Dict:
    """
    Upload data product records to AuroraX

    :param api_key: AuroraX API key
    :type api_key: str
    :param identifier: data source ID
    :type identifier: int
    :param records: data product records to upload
    :type records: List[DataProduct]

    :return: upload response
    :rtype: Dict
    """
    # translate each data product record to a request-friendly dict (ie. convert datetimes to strings, etc.)
    for i, record in enumerate(records):
        if (type(records[i]) is DataProduct):
            records[i] = records[i].to_json_serializable()

    # make request
    url = _aurorax.api.URL_DATA_PRODUCTS_UPLOAD.format(identifier)
    req = _aurorax.AuroraXRequest(url, method="POST", json=records, api_key=api_key)
    res = req.execute()

    # set dict to return
    return_dict = {
        "status_code": res.status_code,
        "data": {},
    }
    if (res.status_code == 400):
        return_dict["data"] = res.data

    # return
    return return_dict



class Location():
    """
    Class representing an AuroraX location (ie. geographic coordinates,
    GSM coordinates, northern/southern magnetic footprints)
    """

    def __init__(self, lat: float, lon: float):
        """
        Constructor

        :param lat: latitude
        :type lat: float
        :param lon: longitude
        :type lon: float
        """
        self.lat = lat
        self.lon = lon

    def __str__(self) -> str:
        """String method

        :return: string format
        :rtype: str
        """
        return str(self.__dict__)

    def __repr__(self) -> str:
        """
        Object representation

        :return: object representation
        :rtype: str
        """
        return "%s(lat=%f, lon=%f)" % (self.__class__.__name__, self.lat, self.lon)



class AuroraXException(Exception):

    def __init__(self, *args, **kwargs):
        response = kwargs.pop("response", None)
        self.response = response
        self.request = kwargs.pop("request", None)
        if (response is not None and not self.request and hasattr(response, "request")):
            self.request = self.response.request
        super(AuroraXException, self).__init__(*args, **kwargs)


class AuroraXRequestsException(AuroraXException):
    pass


class AuroraXUnspecifiedException(AuroraXException):
    pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aurorax']

package_data = \
{'': ['*']}

install_requires = \
['flake8>=3.8.3,<4.0.0', 'humanize>=2.6.0,<3.0.0', 'requests>=2.24.0,<3.0.0']

setup_kwargs = {
    'name': 'pyaurorax',
    'version': '0.2.0',
    'description': 'Python library for interacting with the AuroraX API',
    'long_description': None,
    'author': 'Darren Chaddock',
    'author_email': 'dchaddoc@ucalgary.ca',
    'maintainer': 'Darren Chaddock',
    'maintainer_email': 'dchaddoc@ucalgary.ca',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)



__version__ = "0.2.0"

# classes to be found at the top level
from .api import AuroraXRequest, AuroraXRawRequest, AuroraXResponse
from .location import Location

# pull in exceptions
from aurorax import exceptions

# pull in core modules
from aurorax import api
from aurorax import ephemeris
from aurorax import conjunctions
from aurorax import data_products
from aurorax import availability
from aurorax import requests
from aurorax import sources
from aurorax import location
from aurorax import metadata
from aurorax import util



import aurorax as _aurorax
from typing import Dict as _Dict
from typing import List as _List


def validate(schema: _List, record: _Dict, quiet: bool = False) -> bool:
    """
    Validate the metadata record against a schema. This checks
    that the key names match and there aren't fewer or more keys
    than expected.

    :param schema: metadata schema to validate against
    :type schema: List
    :param record: metadata record to validate
    :type record: Dict

    :return: metadata record is valid
    :rtype: bool
    """
    # check keys
    schema_keys = sorted([i["field_name"] for i in schema])
    record_keys = sorted(record.keys())
    if (schema_keys != record_keys):
        if (quiet is False):
            print("Metadata invalid, keys don't match")
        return False
    return True


def get_ephemeris_schema(identifier: int) -> _List:
    """
    Retrieve the metadata schema for an ephemeris record

    :param identifier: ephemeris source ID
    :type identifier: int

    :return: metadata schema
    :rtype: List
    """
    source_info = _aurorax.sources.get_using_identifier(identifier, format="full_record")
    if ("ephemeris_metadata_schema" in source_info["data"]):
        return source_info["data"]["ephemeris_metadata_schema"]
    else:
        return []


def get_data_products_schema(identifier: int) -> _List:
    """
    Retrieve the metadata schema for a data products record

    :param identifier: ephemeris source ID
    :type identifier: int

    :return: metadata schema
    :rtype: List
    """
    source_info = _aurorax.sources.get_using_identifier(identifier, format="full_record")
    if ("data_products_metadata_schema" in source_info["data"]):
        return source_info["data"]["data_products_metadata_schema"]
    else:
        return []



import aurorax as _aurorax
import datetime as _datetime
import time as _time
from typing import Dict as _Dict

# globals
FIRST_FOLLOWUP_SLEEP_TIME = 0.050  # 50ms
STANDARD_POLLING_SLEEP_TIME = 1.0  # 1s


def get_status(request_url: str) -> _Dict:
    """
    Retrieve the status of a request

    :param request_url: URL of the request information
    :type request_url: str

    :return: status response
    :rtype: Dict
    """
    # get request status
    req = _aurorax.AuroraXRequest(request_url)
    res = req.execute()

    # set return dict
    return_dict = {
        "status_code": res.status_code,
        "data": {},
        "request_status": {
            "completed": False,
            "data_url": ""
        }
    }

    # determine the status of the request
    if (res.status_code == 200):
        return_dict["data"] = res.data
        if (res.data["search_result"]["data_uri"] is not None):
            return_dict["request_status"]["completed"] = True
            return_dict["request_status"]["data_url"] = "%s%s" % (_aurorax.api._URL_API_STUB,
                                                                  res.data["search_result"]["data_uri"])

    # return
    return return_dict


def get_data(data_url: str) -> _Dict:
    """
    Retrieve the data for a request

    :param data_url: URL for the data of a request
    :type data_url: str

    :return: data response
    :rtype: Dict
    """
    # make request
    req = _aurorax.AuroraXRequest(data_url)
    res = req.execute()

    # set return dict
    return_dict = {
        "status_code": res.status_code,
        "data": []
    }
    if (res.status_code == 200):
        return_dict["data"] = res.data["result"]

    # serialize epochs to datetime objects
    for i in range(0, len(return_dict["data"])):
        if ("epoch" in return_dict["data"][i]):
            return_dict["data"][i]["epoch"] = _datetime.datetime.strptime(return_dict["data"][i]["epoch"],
                                                                          "%Y-%m-%dT%H:%M:%S")

    # return
    return return_dict


def get_logs(request_url: str) -> _Dict:
    """
    Retrieve the logs for a request

    :param request_url: URL of the request information
    :type request_url: str

    :return: logs response
    :rtype: Dict
    """
    status = get_status(request_url)
    return_dict = {
        "status_code": status["status_code"],
        "data": [],
    }
    if (status["status_code"] == 200 and "logs" in status["data"]):
        return_dict["data"] = status["data"]["logs"]
    return return_dict


def wait_for_data(request_url: str, poll_interval: float = STANDARD_POLLING_SLEEP_TIME) -> _Dict:
    """
    Block and wait for the data to be made available for a request

    :param request_url: URL of the request information
    :type request_url: str
    :param poll_interval: seconds to wait between polling calls, defaults to STANDARD_POLLING_SLEEP_TIME
    :type poll_interval: float, optional

    :return: status response
    :rtype: Dict
    """
    status = get_status(request_url)
    while (status["request_status"]["completed"] is False):
        _time.sleep(poll_interval)
        status = get_status(request_url)
    return status
